<?php

/**
 * Provides a plugin for connecting to a SearchStax Solr server.
 */
class SearchStaxSolrConnection extends SearchApiSolrConnection {

  /**
   * The "magic" parameter used to switch Solr requests to a different handler.
   */
  public const OVERRIDE_HANDLER_PARAM = '_searchstax_override_handler';

  /**
   * Cached Solr version.
   *
   * @see getSolrVersion()
   */
  protected string $solrVersion;

  /**
   * {@inheritdoc}
   */
  public function ping($timeout = 2) {
    // Copied almost verbatim from the parent class, but we need to use a GET
    // request as HEAD is rejected by some SearchStax apps.
    $start = microtime(TRUE);

    if ($timeout <= 0.0) {
      $timeout = -1;
    }
    $pingUrl = $this->constructUrl(self::PING_SERVLET);

    $response = $this->makeHttpRequest($pingUrl, ['timeout' => $timeout]);

    if ($response->code == 200) {
      // Add 1 µs to the ping time so we never return 0.
      return (microtime(TRUE) - $start) + 1E-6;
    }
    else {
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getLuke($num_terms = 0) {
    if (!empty($this->options['update_token'])) {
      // Unfortunately, SearchStax blocks requests to "admin/luke" for apps with
      // token authentication, so we just return dummy data here to avoid
      // showing an error message to users.
      return [
        'fields' => [],
        'index' => ['numDocs' => -1],
      ];
    }

    return parent::getLuke($num_terms);
  }

  /**
   * {@inheritdoc}
   */
  protected function constructUrl($servlet, array $params = [], $added_query_string = NULL): string {
    if ($servlet === self::SEARCH_SERVLET) {
      $handler_param = self::OVERRIDE_HANDLER_PARAM;
      if (!empty($params[$handler_param])) {
        $servlet = $params[$handler_param];
        unset($params[$handler_param]);
      }
      elseif (preg_match("/(^|&)$handler_param=([^&]+)/", $added_query_string, $match)) {
        $servlet = rawurldecode($match[2]);
        $added_query_string = str_replace($match[0], '', $added_query_string);
        if ($match[1] === '') {
          $added_query_string = ltrim($added_query_string, '&');
        }
      }
    }
    return parent::constructUrl($servlet, $params, $added_query_string);
  }

  /**
   * {@inheritdoc}
   */
  protected function makeHttpRequest($url, array $options = []): object {
    // Add "Authorization" header in case token authentication is enabled.
    if (!empty($this->options['update_token'])) {
      $options['headers']['Authorization'] = "Token {$this->options['update_token']}";
    }

    return parent::makeHttpRequest($url, $options);
  }

  /**
   * {@inheritdoc}
   */
  public function getSolrVersion(): string {
    if (!isset($this->solrVersion)) {
      try {
        $this->solrVersion = parent::getSolrVersion();
      }
      catch (Exception $e) {
        // Some SearchStax apps (specifically those that use token
        // authentication, but probably safer not to hard-code that) block
        // access to the admin endpoints. Fall back to a safe Solr version for
        // those.
        $this->solrVersion = '8.11.1';
      }
    }
    return $this->solrVersion;
  }

}
